%% load image
hd = imread('halfdome.png');
hd = hd(:,:,2);
%% step size
s = 10; % try 1, 3, 10, 100
N = 50000;
%% Run random walk
h = figure;
H = 0;
x = [100;100];
for i = 1:100
    x = cumsum([x(:,end) randn(2,N)], 2);
    H = H + imghist(x, size(hd,1), size(hd,2));
    figure(h); imshow(H * (0.2 * numel(H) / (i*N)));
    title(sprintf('gaussian walk, s = %g, %g iterations', s, i*N));
    drawnow
end
%% Run Metropolis Markov chain
h = figure;
H = 0;
x = [100;100];
for i = 1:20
    x = metro(hd, x(:,end), s, N);
    H = H + imghist(x, size(hd,1), size(hd,2));
    figure(h); imshow(H * (0.2 * numel(H) / (i*N)));
    title(sprintf('metropolis, s = %g, %g iterations', s, i*N));
    drawnow
end
%% Run Metropolis-Hastings Markov chain
h = figure;
H = 0;
x = [100;100];
for i = 1:20
    x = metro_haste(hd, x(:,end), s, N);
    H = H + imghist(x, size(hd,1), size(hd,2));
    figure(h); imshow(H * (0.2 * numel(H) / (i*N)));
    title(sprintf('metropolis-hastings, s = %g, %g iterations', s, i*N));
    drawnow
end
%% Run Veach-style metropolis with weighted contributions
h = figure;
H = zeros(size(hd));
x = [100;100];
for i = 1:20
    [x, H] = metro_veach(hd, x, s, N, H);
    figure(h); imshow(H * (0.2 * numel(H) / (i*N)));
    title(sprintf('metropolis-veach, s = %g, %g iterations', s, i*N));
    drawnow
end
